
(in-package :TRAPS)             ; ***********************************************************
; 
; Created: Tuesday, August 17, 1993 4:37:24 PM
;  OCEMail.p
;  Pascal Interface to the Macintosh Libraries
; 
;   Copyright Apple Computer, Inc. 1990-1993
;   All rights reserved
; 
; ***********************************************************

; $IFC UNDEFINED UsingIncludes
; $SETC UsingIncludes:= 0
; $ENDC

; $IFC NOT UsingIncludes

; $ENDC

; $IFC UNDEFINED UsingOCEMail
; $SETC UsingOCEMail:= 1

; $I+
; $SETC OCEMailIncludes:= UsingIncludes
; $SETC UsingIncludes:= 1

; $IFC UNDEFINED UsingDigitalSignature
; $I DigitalSignature.p
; $ENDC

; $IFC UNDEFINED UsingFiles

(require-interface 'FILES)      ; $I $$Shell(PInterfaces)Files.p
; $ENDC

; $IFC UNDEFINED UsingMemory

(require-interface 'MEMORY)     ; $I $$Shell(PInterfaces)Memory.p
; $ENDC

; $IFC UNDEFINED UsingOCE
; $I OCE.p
; $ENDC

; $IFC UNDEFINED UsingOCEAuthDir
; $I OCEAuthDir.p
; $ENDC

; $IFC UNDEFINED UsingOCEMessaging
; $I OCEMessaging.p
; $ENDC

; $IFC UNDEFINED UsingTextEdit

(require-interface 'TEXTEDIT)   ; $I $$Shell(PInterfaces)TextEdit.p
; $ENDC

; $IFC UNDEFINED UsingTypes

(require-interface 'TYPES)      ; $I $$Shell(PInterfaces)Types.p
; $ENDC

; $SETC UsingIncludes:= OCEMailIncludes

; ************************************************************************************
;  Common Definitions 
; ************************************************************************************

(defrecord MailParamBlockHeader 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)
   )

(def-mactype :MAILMSGREF (find-mactype :SIGNED-LONG));  reference to a new or open letter or message 

(def-mactype :MSAMQUEUEREF (find-mactype :SIGNED-LONG));  reference to an open msam queue 
(def-mactype :MSAMSLOTID (find-mactype :SIGNED-INTEGER));  identifies slots managed by a PMSAM 

(def-mactype :MAILBOXREF (find-mactype :SIGNED-LONG));  reference to an active mailbox 
(def-mactype :MAILSLOTID (find-mactype :SIGNED-INTEGER));  identifies slots within a mailbox 

(defrecord MailSeqNum           ;  identifies a letter in a mailbox 
   (slotID :SIGNED-INTEGER)
   (seqNum :SIGNED-LONG)
   )

;  A MailBuffer is used to describe a buffer used for an IO operation.
; The location of the buffer is pointed to by 'buffer'.
; When reading, the size of the buffer is 'bufferSize'
; and the size of data actually read is 'dataSize'.
; When writing, the size of data to be written is 'bufferSize'
; and the size of data actually written is 'dataSize'.
; 

(defrecord MailBuffer 
   (bufferSize :SIGNED-LONG)
   (buffer :POINTER)
   (dataSize :SIGNED-LONG)
   )

;  A MailReply is used to describe a commonly used reply buffer format.
; It contains a count of tuples followed by an array of tuples.
; The format of the tuple itself depends on each particular call.
; 
(defrecord MailReply 
   (tupleCount :SIGNED-INTEGER)
;  tuple[1..tupleCount] 
   )

;  Shared Memory Communication Area used when Mail Manager sends
; High Level Events to a PMSAM.
; 
(defrecord SMCA 
   (smcaLength :SIGNED-INTEGER) ;  includes size of length field 
   (result :SIGNED-INTEGER)
   (userBytes :SIGNED-LONG)
   (:variant 
      ((slotCID :CREATIONID))   ;  may not be used in some cases 
      ((msgHint :SIGNED-LONG))
      ))

; ************************************************************************************

;  Value of creator and types fields for messages and blocks defined by MailManager 
(defconstant $kMailAppleMailCreator :|apml|);  message and letter block creator 

(defconstant $kMailLtrMsgType :|lttr|);  msg type of letters, reports 
(defconstant $kMailLtrHdrType :|lthd|);  contains letter header 
(defconstant $kMailContentType :|body|);  contains content of letter 
(defconstant $kMailEnclosureListType :|elst|);  contains list of enclosures 
(defconstant $kMailEnclosureDesktopType :|edsk|);  contains desktop mgr info for enclosures 

(defconstant $kMailEnclosureFileType :|asgl|);  contains a file enclosure 
;  format is defined by AppleSingle 

(defconstant $kMailImageBodyType :|imag|);  contains image of letter 
; 		format is struct TPfPgDir		 - in Printing.h
; 		TPfPgDir = RECORD
; 			pageCount: INTEGER;			- number of pages in the image.
; 			iPgPos: LONGINT[1..129];	- iPgPos[n] is the offset from the start of the block
; 			END;
; 										- to image of page n.
; 										- iPgPos[n+1] - iPgPos[n] is the length of page n.
; 

(defconstant $kMailMSAMType :|gwyi|);  contains msam specific information 

(defconstant $kMailTunnelLtrType :|tunl|);  used to read a tunnelled message 
(defconstant $kMailHopInfoType :|hopi|);  used to read hopInfo for a tunnelled message 

(defconstant $kMailReportType :|rpti|);  contains report info 
; 
; Reports have the isReport bit set in MailIndications and contain a block of type kMailReport.
; This block has a header, IPMReportBlockHeader,
; followed by an array of elements, each of type IPMRecipientReport
; 

;  Various families used by mail or related msgs 
(defconstant $kMailFamily :|mail|);  Defines family of "mail" msgs: content, header, etc 
(defconstant $kMailFamilyFile :|file|);  Defines family of "direct display" msgs 

; ************************************************************************************

(def-mactype :MAILATTRIBUTEID (find-mactype :SIGNED-INTEGER))

;  Values of MailAttributeID 
;  Message store attributes - stored in the catalog 
;  Will always be present in a letter and have fixed sizes 
(defconstant $kMailLetterFlagsBit 1);  MailLetterFlags 

;  Letter attributes - stored in the letter 
;  Will always be present in a letter and have fixed sizes 
(defconstant $kMailIndicationsBit 3);  MailIndications 
(defconstant $kMailMsgTypeBit 4);  OCECreatorType 
(defconstant $kMailLetterIDBit 5);  MailLetterID 
(defconstant $kMailSendTimeStampBit 6);  MailTime 
(defconstant $kMailNestingLevelBit 7);  MailNestingLevel 
(defconstant $kMailMsgFamilyBit 8);  OSType 

;  Letter attributes - stored in the letter 
;  May be present in a letter and have fixed sizes 
(defconstant $kMailReplyIDBit 9);  MailLetterID 
(defconstant $kMailConversationIDBit 10);  MailLetterID 

;  Letter attributes - stored in the letter 
;  May be present in a letter and have variable length sizes 
(defconstant $kMailSubjectBit 11);  RString 
(defconstant $kMailFromBit 12)  ;  MailRecipient 
(defconstant $kMailToBit 13)    ;  MailRecipient 
(defconstant $kMailCcBit 14)    ;  MailRecipient 
(defconstant $kMailBccBit 15)   ;  MailRecipient 

(def-mactype :MAILATTRIBUTEMASK (find-mactype :SIGNED-LONG))

;  Values of MailAttributeMask 
(defconstant $kMailLetterFlagsMask #X1); 1<<(kMailLetterFlagsBit-1)
(defconstant $kMailIndicationsMask #X4); 1<<(kMailIndicationsBit-1)
(defconstant $kMailMsgTypeMask #X8); 1<<(kMailMsgTypeBit-1)
(defconstant $kMailLetterIDMask #X10); 1<<(kMailLetterIDBit-1)
(defconstant $kMailSendTimeStampMask #X20); 1<<(kMailSendTimeStampBit-1)
(defconstant $kMailNestingLevelMask #X40); 1<<(kMailNestingLevelBit-1)
(defconstant $kMailMsgFamilyMask #X80); 1<<(kMailMsgFamilyBit-1)
(defconstant $kMailReplyIDMask #X100); 1<<(kMailReplyIDBit-1)
(defconstant $kMailConversationIDMask #X200); 1<<(kMailConversationIDBit-1)
(defconstant $kMailSubjectMask #X400); 1<<(kMailSubjectBit-1)
(defconstant $kMailFromMask #X800); 1<<(kMailFromBit-1)
(defconstant $kMailToMask #X1000); 1<<(kMailToBit-1)
(defconstant $kMailCcMask #X2000); 1<<(kMailCcBit-1)
(defconstant $kMailBccMask #X4000); 1<<(kMailBccBit-1)

(defrecord MailAttributeBitmap 
   (reservedA :UNSIGNED-INTEGER);  16 bits 
   (reservedB :UNSIGNED-INTEGER)
   (bcc :UNSIGNED-INTEGER)      ;  BOOLEAN 
   (cc :UNSIGNED-INTEGER)       ;  BOOLEAN 
   (toRecipient :UNSIGNED-INTEGER);  BOOLEAN 
   (from :UNSIGNED-INTEGER)     ;  BOOLEAN 
   (subject :UNSIGNED-INTEGER)  ;  BOOLEAN 
   (conversationID :UNSIGNED-INTEGER);  BOOLEAN 
   (replyID :UNSIGNED-INTEGER)  ;  BOOLEAN 
   (msgFamily :UNSIGNED-INTEGER);  BOOLEAN 
   (nestingLevel :UNSIGNED-INTEGER);  BOOLEAN 
   (sendTimeStamp :UNSIGNED-INTEGER);  BOOLEAN 
   (letterID :UNSIGNED-INTEGER) ;  BOOLEAN 
   (msgType :UNSIGNED-INTEGER)  ;  BOOLEAN 
   (indications :UNSIGNED-INTEGER);  BOOLEAN 
   (reservedC :UNSIGNED-INTEGER);  BOOLEAN 
   (letterFlags :UNSIGNED-INTEGER);  BOOLEAN 
   )

; ************************************************************************************
(def-mactype :MAILLETTERSYSTEMFLAGS (find-mactype :SIGNED-INTEGER))

;  Values of MailLetterSystemFlags 
(defconstant $kMailIsLocalBit 2);  letter is available locally (either by nature or via cache) 

(defconstant $kMailIsLocalMask #X4); 1<<kMailIsLocalBit

(def-mactype :MAILLETTERUSERFLAGS (find-mactype :SIGNED-INTEGER))

(defconstant $kMailReadBit 0)   ;  this letter has been opened 
(defconstant $kMailDontArchiveBit 1);  this letter is not 
;  to be archived either because 
;  it has already been archived or 
;  it should not be archived. 
(defconstant $kMailInTrashBit 2);  this letter is in trash 

;  Values of MailLetterUserFlags 
(defconstant $kMailReadMask #X1); 1<<kMailReadBit
(defconstant $kMailDontArchiveMask #X2); 1<<kMailDontArchiveBit
(defconstant $kMailInTrashMask #X4); 1<<kMailInTrashBit

(defrecord MailLetterFlags 
   (sysFlags :SIGNED-INTEGER)
   (userFlags :SIGNED-INTEGER)
   )

(defrecord MailMaskedLetterFlags 
   (flagMask :MAILLETTERFLAGS)  ;  flags that are to be set 
   (flagValues :MAILLETTERFLAGS);  and their values 
   )

(defconstant $kMailOriginalInReportBit 1)
(defconstant $kMailNonReceiptReportsBit 3)
(defconstant $kMailReceiptReportsBit 4)
(defconstant $kMailForwardedBit 5)
(defconstant $kMailPriorityBit 6)
(defconstant $kMailIsReportWithOriginalBit 8)
(defconstant $kMailIsReportBit 9)
(defconstant $kMailHasContentBit 10)
(defconstant $kMailHasSignatureBit 11)
(defconstant $kMailAuthenticatedBit 12)
(defconstant $kMailSentBit 13)

;  Values of MailIndications 
(defconstant $kMailSentMask #X1000); 1<<(kMailSentBit-1)
(defconstant $kMailAuthenticatedMask #X800); 1<<(kMailAuthenticatedBit-1)
(defconstant $kMailHasSignatureMask #X400); 1<<(kMailHasSignatureBit-1)
(defconstant $kMailHasContentMask #X200); 1<<(kMailHasContentBit-1)
(defconstant $kMailIsReportMask #X100); 1<<(kMailIsReportBit-1)
(defconstant $kMailIsReportWithOriginalMask #X80); 1<<(kMailIsReportWithOriginalBit-1)
(defconstant $kMailPriorityMask #X60); 3<<(kMailPriorityBit-1)
(defconstant $kMailForwardedMask #X10); 1<<(kMailForwardedBit-1)
(defconstant $kMailReceiptReportsMask #X8); 1<<(kMailReceiptReportsBit-1)
(defconstant $kMailNonReceiptReportsMask #X4); 1<<(kMailNonReceiptReportsBit-1)
(defconstant $kMailOriginalInReportMask #X3); 3<<kMailOriginalInReportBit-1)

(defrecord MailIndications 
   (reservedB :UNSIGNED-INTEGER);  16 bits 
   (hasStandardContent :UNSIGNED-INTEGER);  this letter has standard interchange content 
   (hasImageContent :UNSIGNED-INTEGER);  this letter has a standard image 
   (hasNativeContent :UNSIGNED-INTEGER);  this letter has a main enclosure 
   (sent :UNSIGNED-INTEGER)     ;  this letter was sent, not just composed 
   (authenticated :UNSIGNED-INTEGER);  this letter was created and transported with authentication 
   (hasSignature :UNSIGNED-INTEGER);  this letter was signed with a digital signature 
   (hasContent :UNSIGNED-INTEGER);  this letter has content 
   (isReport :UNSIGNED-INTEGER) ;  this letter is really a report 
   (isReportWithOriginal :UNSIGNED-INTEGER);  this report contains the original letter 

   (priority :UNSIGNED-INTEGER) ;  Bits defined as in IPMPriority 
;  the originator has: 
   (forwarded :UNSIGNED-INTEGER); 		forwarded this letter 

;  the originator requests: 
   (receiptReports :UNSIGNED-INTEGER); 		delivery reports 
   (nonReceiptReports :UNSIGNED-INTEGER); 		non-delivery reports 
   (originalInReport :UNSIGNED-INTEGER); 		original letter to be enclosed in reports 
   )

;  values of the field originalInReport in MailIndications 
(defconstant $kMailNoOriginal 0);  do not enclose original in reports 
(defconstant $kMailEncloseOnNonReceipt 3);  enclose original in non-delivery reports 

(def-mactype :MAILLETTERID (find-mactype :IPMMSGID))

(defrecord MailTime 
   (time :SIGNED-LONG)          ;  current UTC(GMT) time 
   (offset :SIGNED-LONG)        ;  offset from GMT 
   )

(def-mactype :MAILNESTINGLEVEL (find-mactype :SIGNED-INTEGER));  innermost letter has nestingLevel 0 

(def-mactype :MAILRECIPIENT (find-mactype :OCERECIPIENT))

; ************************************************************************************

(defconstant $kMailTextSegmentBit 0)
(defconstant $kMailPictSegmentBit 1)
(defconstant $kMailSoundSegmentBit 2)
(defconstant $kMailStyledTextSegmentBit 3)
(defconstant $kMailMovieSegmentBit 4)

(def-mactype :MAILSEGMENTMASK (find-mactype :SIGNED-INTEGER))

;  Values of MailSegmentMask 
(defconstant $kMailTextSegmentMask #X1); 1<<kMailTextSegmentBit
(defconstant $kMailPictSegmentMask #X2); 1<<kMailPictSegmentBit
(defconstant $kMailSoundSegmentMask #X4); 1<<kMailSoundSegmentBit
(defconstant $kMailStyledTextSegmentMask #X8); 1<<kMailStyledTextSegmentBit
(defconstant $kMailMovieSegmentMask #X10); 1<<kMailMovieSegmentBit

(def-mactype :MAILSEGMENTTYPE (find-mactype :SIGNED-INTEGER))

;  Values of MailSegmentType 
(defconstant $kMailInvalidSegmentType 0)
(defconstant $kMailTextSegmentType 1)
(defconstant $kMailPictSegmentType 2)
(defconstant $kMailSoundSegmentType 3)
(defconstant $kMailStyledTextSegmentType 4)
(defconstant $kMailMovieSegmentType 5)

; ************************************************************************************
(defconstant $kMailErrorLogEntryVersion #X101)
(defconstant $kMailMSAMErrorStringListID 128);  These 'STR#' resources should be 
(defconstant $kMailMSAMActionStringListID 129);  in the PMSAM resource fork 

(def-mactype :MAILLOGERRORTYPE (find-mactype :SIGNED-INTEGER))

;  Values of MailLogErrorType 
(defconstant $kMailELECorrectable 0)
(defconstant $kMailELEError 1)
(defconstant $kMailELEWarning 2)
(defconstant $kMailELEInformational 3)

(def-mactype :MAILLOGERRORCODE (find-mactype :SIGNED-INTEGER))

;  Values of MailLogErrorCode 
;  positive codes are indices into 
;  PMSAM defined STRINGs 
(defconstant $kMailMSAMErrorCode 0)
(defconstant $kMailMiscError -1);  negative codes are OCE defined 
(defconstant $kMailNoModem -2)  ;  modem required, but missing 

(defrecord MailErrorLogEntryInfo ;  < 128 bytes 
   (version :SIGNED-INTEGER)
   (timeOccurred :SIGNED-LONG)  ;  do not fill in 
   (reportingPMSAM (:STRING 31));  do not fill in 
   (reportingMSAMSlot (:STRING 31));  do not fill in 
   (errorType :SIGNED-INTEGER)
   (errorCode :SIGNED-INTEGER)
   (errorResource :SIGNED-INTEGER);  resources are valid if 
   (actionResource :SIGNED-INTEGER);  errorCode = kMailMSAMErrorCode 
;  index starts from 1 
   (filler :SIGNED-LONG)
   (filler2 :SIGNED-INTEGER)
   )

; ************************************************************************************
(def-mactype :MAILBLOCKMODE (find-mactype :SIGNED-INTEGER))

;  Values of MailBlockMode 
(defconstant $kMailFromStart 1) ;  write data from offset calculated from 
(defconstant $kMailFromLEOB 2)  ;  start of block, end of block, 
(defconstant $kMailFromMark 3)  ;  or from the current mark 

(defrecord MailEnclosureInfo 
   (enclosureName (:POINTER (:STRING 255)))
   (catInfo :CINFOPBPTR)
   (comment (:POINTER (:STRING 255)))
   (icon :POINTER)
   )

; ************************************************************************************

(defconstant $kOCESetupLocationNone 0);  disconnect state 
(defconstant $kOCESetupLocationMax 8);  maximum location value 

(def-mactype :OCESETUPLOCATION (find-mactype :UNSIGNED-BYTE))

;  location state is a bitmask, 0x1=>1st location active, 
;   0x2 => 2nd, 0x4 => 3rd, etc.
; 

; 
; #define MailLocationMask(locationNumber) (1<<((locationNumber)-1))
; 

(def-mactype :MAILLOCATIONFLAGS (find-mactype :UNSIGNED-BYTE))

(defrecord MailLocationInfo 

; ERROR!! Record field LOCATION declared Non-PACKED BYTE !
   (location :UNSIGNED-BYTE)

; ERROR!! Record field ACTIVE declared Non-PACKED BYTE !
   (active :UNSIGNED-BYTE)
   )

; ************************************************************************************
;  Definitions for Personal MSAMs 
; ************************************************************************************

(defconstant $kMailEPPCMsgVersion 3)

(defrecord MailEPPCMsg 
   (version :SIGNED-INTEGER)
   (:variant 
      ((theSMCA (:POINTER :SMCA)));  for 'crsl', 'mdsl', 'dlsl', 'sndi', 'msgo', 'admn' 
      ((sequenceNumber :SIGNED-LONG));  for 'inqu', 'dlom' 
      ((locationInfo :MAILLOCATIONINFO));  for 'locc' 
      ))

;  Values of OCE defined High Level Event message classes 
(defconstant $kMailEPPCCreateSlot :|crsl|)
(defconstant $kMailEPPCModifySlot :|mdsl|)
(defconstant $kMailEPPCDeleteSlot :|dlsl|)
(defconstant $kMailEPPCShutDown :|quit|)
(defconstant $kMailEPPCMailboxOpened :|mbop|)
(defconstant $kMailEPPCMailboxClosed :|mbcl|)
(defconstant $kMailEPPCMsgPending :|msgp|)
(defconstant $kMailEPPCSendImmediate :|sndi|)
(defconstant $kMailEPPCContinue :|cont|)
(defconstant $kMailEPPCSchedule :|sked|)
(defconstant $kMailEPPCAdmin :|admn|)
(defconstant $kMailEPPCInQUpdate :|inqu|)
(defconstant $kMailEPPCMsgOpened :|msgo|)
(defconstant $kMailEPPCDeleteOutQMsg :|dlom|)
(defconstant $kMailEPPCWakeup :|wkup|)
(defconstant $kMailEPPCLocationChanged :|locc|)

(defrecord MailTimer 
   (:variant 
      ((frequency :SIGNED-LONG));  how often to connect (secs) 
      ((connectTime :SIGNED-LONG));  time since midnight (secs) 
      ))

(defconstant $kMailTimerOff 0)  ;  control is off 
(defconstant $kMailTimerTime 1) ;  specifies connect time (relative to midnight) 
(defconstant $kMailTimerFrequency 2);  specifies connect frequency 

(def-mactype :MAILTIMERKIND (find-mactype :UNSIGNED-BYTE))

(defrecord MailTimers 
   (sendTimeKind :UNSIGNED-BYTE);  either kMailTimerTime or kMailTimerFrequency 
   (receiveTimeKind :UNSIGNED-BYTE);  either kMailTimerTime or kMailTimerFrequency 
   (send :MAILTIMER)
   (receive :MAILTIMER)
   )

(defrecord MailStandardSlotInfoAttribute 
   (version :SIGNED-INTEGER)
   (active :UNSIGNED-BYTE)      ;  active if MailLocationMask(i) is set 
   (padByte :UNSIGNED-BYTE)
   (sendReceiveTimer :MAILTIMERS)
   )

(defrecord PMSAMGetMSAMRecordPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (msamCID :CREATIONID)
   )

(defrecord PMSAMOpenQueuesPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (inQueueRef :SIGNED-LONG)
   (outQueueRef :SIGNED-LONG)
   (MSAMSlotID :SIGNED-INTEGER)
   (filler (:ARRAY :SIGNED-LONG 2))
   )

(def-mactype :PMSAMSTATUS (find-mactype :SIGNED-INTEGER))

;  Values of PMSAMStatus 
(defconstant $kPMSAMStatusPending 1);  for inQueue and outQueue 
(defconstant $kPMSAMStatusError 2);  for inQueue and outQueue 
(defconstant $kPMSAMStatusSending 3);  for outQueue only 
(defconstant $kPMSAMStatusCaching 4);  for inQueue only 
(defconstant $kPMSAMStatusSent 5);  for outQueue only 

(defrecord PMSAMSetStatusPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)
   (seqNum :SIGNED-LONG)
   (msgHint :SIGNED-LONG)       ;  for posting cache error,set this to 0 when report outq status 
   (status :SIGNED-INTEGER)
   )

(defrecord PMSAMLogErrorPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MSAMSlotID :SIGNED-INTEGER) ;  0 for PMSAM errors 
   (logEntry (:POINTER :MAILERRORLOGENTRYINFO))
   (filler (:ARRAY :SIGNED-LONG 2))
   )

; **************************************************************************************

(defconstant $kMailMsgSummaryVersion 1)

(defrecord MailMasterData 
   (attrMask :MAILATTRIBUTEBITMAP);  indicates attributes present in MsgSummary 
   (messageID :IPMMSGID)
   (replyID :IPMMSGID)
   (conversationID :IPMMSGID)
   )

;  Values for addressedToMe in MailCoreData record 

(defconstant $kAddressedAs_TO #X1)
(defconstant $kAddressedAs_CC #X2)
(defconstant $kAddressedAs_BCC #X4)

(defrecord MailCoreData 
   (letterFlags :MAILLETTERFLAGS)
   (messageSize :SIGNED-LONG)
   (letterIndications :MAILINDICATIONS)
   (messageType :OCECREATORTYPE)
   (sendTime :MAILTIME)
   (messageFamily :OSTYPE)

; ERROR!! Record field RESERVED declared Non-PACKED BYTE !
   (reserved :UNSIGNED-BYTE)

; ERROR!! Record field ADDRESSEDTOME declared Non-PACKED BYTE !
   (addressedToMe :UNSIGNED-BYTE)
   (agentInfo (:ARRAY :UNSIGNED-BYTE (- 6 1 -1)));  6 bytes of special info [set to zero] 
;  these are variable length and even padded 
   (sender :RSTRING32)          ;  recipient's entityName (trunc) 
   (subject :RSTRING32)         ;  subject maybe truncated 
   )

(defrecord MSAMMsgSummary 
   (version :SIGNED-INTEGER)    ;  following flags are defaulted by Toolbox 
   (msgDeleted :BOOLEAN)        ;  true if msg is to be deleted by PMSAM 
   (msgUpdated :BOOLEAN)        ;  true if msgSummary was updated by MailManager 
   (msgCached :BOOLEAN)         ;  true if msg is in the slot's InQueue 
; padByte: Byte;
   (masterData :MAILMASTERDATA)
   (coreData :MAILCOREDATA)
   )

(defconstant $kMailMaxPMSAMMsgSummaryData 128);  PMSAM can put up to 128 bytes of private msg summary data 

(defrecord PMSAMCreateMsgSummaryPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (inQueueRef :SIGNED-LONG)
   (seqNum :SIGNED-LONG)        ;  <- seq of the new message 
   (msgSummary (:POINTER :MSAMMSGSUMMARY));  attributes and mask filled in 
   (buffer (:POINTER :MAILBUFFER));  PMSAM specific data to be appended 
   )

(defrecord PMSAMPutMsgSummaryPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (inQueueRef :SIGNED-LONG)
   (seqNum :SIGNED-LONG)
   (letterFlags (:POINTER :MAILMASKEDLETTERFLAGS));  if not nil, then set letterFlags 
   (buffer (:POINTER :MAILBUFFER));  PMSAM specific data to be overwritten 
   )

(defrecord PMSAMGetMsgSummaryPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (inQueueRef :SIGNED-LONG)
   (seqNum :SIGNED-LONG)
   (msgSummary (:POINTER :MSAMMSGSUMMARY));  if not nil, then read in the msgSummary 
   (buffer (:POINTER :MAILBUFFER));  PMSAM specific data to be read 
   (msgSummaryOffset :SIGNED-INTEGER);  offset of PMSAM specific data  
;  from start of MsgSummary 
   )

; **************************************************************************************
;  Definitions for Server MSAMs 
; ************************************************************************************

(def-mactype :SMSAMADMINCODE (find-mactype :SIGNED-INTEGER))

;  Values of SMSAMAdminCode 
(defconstant $kSMSAMNotifyFwdrSetupChange 1)
(defconstant $kSMSAMNotifyFwdrNameChange 2)
(defconstant $kSMSAMNotifyFwdrPwdChange 3)
(defconstant $kSMSAMGetDynamicFwdrParams 4)

(def-mactype :SMSAMSLOTCHANGES (find-mactype :SIGNED-LONG))

(defconstant $kSMSAMFwdrHomeInternetChangedBit 0)
(defconstant $kSMSAMFwdrConnectedToChangedBit 1)
(defconstant $kSMSAMFwdrForeignRLIsChangedBit 2)
(defconstant $kSMSAMFwdrMnMServerChangedBit 3)

;  Values of SMSAMSlotChanges 
(defconstant $kSMSAMFwdrEverythingChangedMask -1)
(defconstant $kSMSAMFwdrHomeInternetChangedMask #X1); 1<<kSMSAMFwdrHomeInternetChangedBit
(defconstant $kSMSAMFwdrConnectedToChangedMask #X2); 1<<kSMSAMFwdrConnectedToChangedBit
(defconstant $kSMSAMFwdrForeignRLIsChangedMask #X4); 1<<kSMSAMFwdrForeignRLIsChangedBit
(defconstant $kSMSAMFwdrMnMServerChangedMask #X8); 1<<kSMSAMFwdrMnMServerChangedBit

(defrecord SMSAMSetupChange     ;  kSMSAMNotifyFwdrSetupChange 
   (whatChanged :SIGNED-LONG)   ;   --> bitmap of what parameters changed 
   (serverHint :ADDRBLOCK)      ;   --> try this ADAP server first 
   )

(defrecord SMSAMNameChange      ;  kSMSAMNotifyFwdrNameChange 
   (newName :RSTRING)           ;   --> msams new name 
   (serverHint :ADDRBLOCK)      ;   --> try this ADAP server first 
   )

(defrecord SMSAMPasswordChange  ;  kSMSAMNotifyFwdrPasswordChange 
   (newPassword :RSTRING)       ;   --> msams new password 
   (serverHint :ADDRBLOCK)      ;   --> try this ADAP server first 
   )

(defrecord SMSAMDynamicParams   ;  kSMSAMGetDynamicFwdrParams 
   (curDiskUsed :SIGNED-LONG)   ;  <--  amount of disk space used by msam 
   (curMemoryUsed :SIGNED-LONG) ;  <--  amount of memory used by msam 
   )

(defrecord SMSAMAdminEPPCRequest 
   (adminCode :SIGNED-INTEGER)
   (:variant 
      ((setupChange :SMSAMSETUPCHANGE))
      ((nameChange :SMSAMNAMECHANGE))
      ((passwordChange :SMSAMPASSWORDCHANGE))
      ((dynamicParams :SMSAMDYNAMICPARAMS))
      ))

(defrecord SMSAMSetupPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (serverMSAM (:POINTER :RECORDID))
   (password (:POINTER :RSTRING))
   (gatewayType :OSTYPE)
   (gatewayTypeDescription (:POINTER :RSTRING))
   (catalogServerHint :ADDRBLOCK)
   )

(defrecord SMSAMStartupPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (msamIdentity :SIGNED-LONG)
   (queueRef :SIGNED-LONG)
   )

(defrecord SMSAMShutdownPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)
   )

; **************************************************************************************
;  Definitions for reading and writing MSAM Letters 
; **************************************************************************************

(defrecord MSAMEnumeratePB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)
   (startSeqNum :SIGNED-LONG)
   (nextSeqNum :SIGNED-LONG)
   (buffer :MAILBUFFER)
;  buffer contains a Mail Reply. Each tuple is a
; 		MSAMEnumerateInQReply when enumerating the inQueue
; 		MSAMEnumerateOutQReply when enumerating the outQueue
; 	
   )

(defrecord MSAMEnumerateInQReply 
   (seqNum :SIGNED-LONG)
   (msgDeleted :BOOLEAN)        ;  true if msg is to be deleted by PMSAM 
   (msgUpdated :BOOLEAN)        ;  true if MsgSummary has been updated by TB 
   (msgCached :BOOLEAN)         ;  true if msg is in the incoming queue 
; padByte: Byte;
   )

(defrecord MSAMEnumerateOutQReply 
   (seqNum :SIGNED-LONG)

; ERROR!! Record field DONE declared PACKED BOOLEAN !
   (done :BOOLEAN)              ;  true if all responsible recipients have been processed 
   (priority :IPMPRIORITY)
   (approxSize :SIGNED-LONG)

; ERROR!! Record field TUNNELFORM declared PACKED BOOLEAN !
   (tunnelForm :BOOLEAN)        ;  true if this letter has to be tunnelled 
   (padByte :UNSIGNED-BYTE)
   (nextHop :NETWORKSPEC)       ;  valid if tunnelForm is true 
   (msgType :OCECREATORTYPE)    ;  valid if tunnelForm is true 
   )

(defrecord MSAMDeletePB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)
   (seqNum :SIGNED-LONG)

; ERROR!! Record field MSGONLY declared PACKED BOOLEAN !
   (msgOnly :BOOLEAN)           ;  only valid for PMSAM & inQueue 
;  set true to delete message but not msgSummary 
   (padByte :UNSIGNED-BYTE)
   (result :SIGNED-INTEGER)     ;  only valid for SMSAM & tunnelled messages 
;  Values for result:
; 											kIPMNoResponsibility
; 											kIPMNoInformation
; 											kIPMRecMaybeNotSent
; 											kIPMRecMaybeSent
; 											
   )

(defrecord MSAMOpenPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)
   (seqNum :SIGNED-LONG)
   (MailMsgRef :SIGNED-LONG)
   )

(defrecord MSAMOpenNestedPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (nestedRef :SIGNED-LONG)
   )

(defrecord MSAMClosePB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   )

(defrecord MSAMGetMsgHeaderPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (selector :IPMHEADERSELECTOR)
   (offset :SIGNED-LONG)
   (buffer :MAILBUFFER)
   (remaining :SIGNED-LONG)
   )

(defrecord MSAMGetAttributesPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (requestMask :MAILATTRIBUTEBITMAP);  kMailIndicationsBit thru kMailSubjectBit 
   (buffer :MAILBUFFER)
; 	buffer returned will contain the attribute values of
; 		the attributes indicated in responseMask,
; 		from the attribute indicated by the least significant bit set
; 		to the attribute indicated by the most significant bit set.
; 		Note that recipients - from, to, cc, bcc cannot be read using
; 		this call. Use GetRecipients to read these.
; 	
   (responseMask :MAILATTRIBUTEBITMAP)
   (more :BOOLEAN)
   )

(defconstant $kMailResolvedList 0);  attrID value to get resolved recipient list 

(defrecord MailOriginalRecipient 
   (index :SIGNED-INTEGER)
;  Followed by OCEPackedRecipient 
   )

(defrecord MailResolvedRecipient 
   (index :SIGNED-INTEGER)
   (recipientFlags :SIGNED-INTEGER)

; ERROR!! Record field RESPONSIBLE declared PACKED BOOLEAN !
   (responsible :BOOLEAN)
   (padByte :UNSIGNED-BYTE)
;  Followed by OCEPackedRecipient 
   )

(defrecord MSAMGetRecipientsPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (attrID :SIGNED-INTEGER)     ;  kMailFromBit thru kMailBccBit 
   (startIndex :SIGNED-INTEGER) ;  starts at 1 
   (buffer :MAILBUFFER)
;  	buffer contains a Mail Reply. Each tuple is a
; 		MailOriginalRecipient if getting original recipients
; 								ie the attrID is kMail[From, To, Cc, Bcc]Bit
; 		MailResolvedRecipient if getting resolved reicpients
; 								ie the attrID is kMailResolvedList
; 		Both tuples are word alligned.
; 	
   (nextIndex :SIGNED-INTEGER)
   (more :BOOLEAN)
   )

(defrecord MSAMGetContentPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (segmentMask :SIGNED-INTEGER)
   (buffer :MAILBUFFER)
   (textScrap (:POINTER :STSCRPREC))
   (script :SIGNED-INTEGER)
   (segmentType :SIGNED-INTEGER)
   (endOfScript :BOOLEAN)
   (endOfSegment :BOOLEAN)
   (endOfContent :BOOLEAN)
   (segmentLength :SIGNED-LONG) ;  NEW: <-  valid first call in a segment 
   (segmentID :SIGNED-LONG)     ;  NEW: <-> identifier for this segment 
   )

(defrecord MSAMGetEnclosurePB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)

; ERROR!! Record field CONTENTENCLOSURE declared PACKED BOOLEAN !
   (contentEnclosure :BOOLEAN)
   (padByte :UNSIGNED-BYTE)
   (buffer :MAILBUFFER)

; ERROR!! Record field ENDOFFILE declared PACKED BOOLEAN !
   (endOfFile :BOOLEAN)

; ERROR!! Record field ENDOFENCLOSURES declared PACKED BOOLEAN !
   (endOfEnclosures :BOOLEAN)
   )

(defrecord MailBlockInfo 
   (blockType :OCECREATORTYPE)
   (offset :SIGNED-LONG)
   (blockLength :SIGNED-LONG)
   )

(defrecord MSAMEnumerateBlocksPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (startIndex :SIGNED-INTEGER) ;  starts at 1 
   (buffer :MAILBUFFER)
;  	buffer contains a Mail Reply. Each tuple is a MailBlockInfo 
   (nextIndex :SIGNED-INTEGER)
   (more :BOOLEAN)
   )

(defrecord MSAMGetBlockPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (blockType :OCECREATORTYPE)
   (blockIndex :SIGNED-INTEGER)
   (buffer :MAILBUFFER)
   (dataOffset :SIGNED-LONG)

; ERROR!! Record field ENDOFBLOCK declared PACKED BOOLEAN !
   (endOfBlock :BOOLEAN)
   (padByte :UNSIGNED-BYTE)
   (remaining :SIGNED-LONG)
   )

;  YOU SHOULD BE USING THE NEW FORM OF MARK RECIPIENTS
;   THIS VERSION IS MUCH SLOWER AND KEPT FOR COMPATIBILITY
;   REASONS.
; 

(defrecord MSAMMarkRecipientsPB ;  not valid for tunnel form letters 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)
   (seqNum :SIGNED-LONG)
   (buffer :MAILBUFFER)
;  	buffer contains a Mail Reply. Each tuple is an unsigned short,
; 		the index of a recipient to be marked. 
   )

; 
;   same as MSAMMarkRecipients except it takes a mailMsgRef instead of 
;   queueRef, seqNum 
; 

(defrecord MSAMnMarkRecipientsPB ;  not valid for tunnel form letters 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (buffer :MAILBUFFER)
;  	buffer contains a Mail Reply. Each tuple is an unsigned short,
; 		the index of a recipient to be marked. 
   )

; **************************************************************************************
(defrecord MSAMCreatePB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)
   (asLetter :BOOLEAN)          ;  indicate if we should create as letter or msg 
   (msgType :IPMMSGTYPE)        ;  is msg creator/type if msg, mail type creator o.w. 
;  default for mail should be: kMailAppleMailCreator, 
;  kMailLtrMsgType 
;  OLD: type lttr indicates create letter 
   (refCon :SIGNED-LONG)        ;  for messages only 
   (seqNum :SIGNED-LONG)        ;  set if creating message in the inQueue 
   (tunnelForm :BOOLEAN)        ;  if true tunnelForm else newForm 
   (bccRecipients :BOOLEAN)     ;  true if creating letter with bcc recipients 
   (newRef :SIGNED-LONG)
   )

(defrecord MSAMBeginNestedPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (refCon :SIGNED-LONG)        ;  for messages only 
   (msgType :IPMMSGTYPE)
   )

(defrecord MSAMEndNestedPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   )

(defrecord MSAMSubmitPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)

; ERROR!! Record field SUBMITFLAG declared PACKED BOOLEAN !
   (submitFlag :BOOLEAN)
   (padByte :UNSIGNED-BYTE)
   (msgID :IPMMSGID)
   )

(defrecord MSAMPutMsgHeaderPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (replyQueue (:POINTER :OCERECIPIENT))
   (sender (:POINTER :IPMSENDER))
   (deliveryNotification :IPMNOTIFICATIONTYPE)
   (priority :IPMPRIORITY)
   )

(defrecord MSAMPutAttributePB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (attrID :SIGNED-INTEGER)     ;  kMailIndicationsBit thru kMailSubjectBit 
   (buffer :MAILBUFFER)
   )

(defrecord MSAMPutRecipientPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (attrID :SIGNED-INTEGER)     ;  kMailFromBit thru kMailBccBit 
   (recipient (:POINTER :OCERECIPIENT))
   (responsible :BOOLEAN)       ;  valid for server and message msams only 
   )

(defrecord MSAMPutContentPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (segmentType :SIGNED-INTEGER)

; ERROR!! Record field APPEND declared PACKED BOOLEAN !
   (append :BOOLEAN)
   (padByte :UNSIGNED-BYTE)
   (buffer :MAILBUFFER)
   (textScrap (:POINTER :STSCRPREC))

; ERROR!! Record field STARTNEWSCRIPT declared PACKED BOOLEAN !
   (startNewScript :BOOLEAN)
   (script :SIGNED-INTEGER)     ;  valid only if startNewScript is true 
   )

(defrecord MSAMPutEnclosurePB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (contentEnclosure :BOOLEAN)
   (padByte :BOOLEAN)
   (hfs :BOOLEAN)               ;  true => in file system, false => in memory 
   (append :BOOLEAN)
   (buffer :MAILBUFFER)         ;  Unused if hfs == true 
   (enclosure :FSSPEC)
   (addlInfo :MAILENCLOSUREINFO)
   )

(defrecord MSAMPutBlockPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (refCon :SIGNED-LONG)        ;  for messages only 
   (blockType :OCECREATORTYPE)
   (append :BOOLEAN)
   (buffer :MAILBUFFER)
   (mode :SIGNED-INTEGER)       ;  if blockType is kMailTunnelLtrType or kMailHopInfoType 
;  mode is assumed to be kMailFromMark 
   (offset :SIGNED-LONG)
   )

; **************************************************************************************
(defrecord MSAMCreateReportPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (queueRef :SIGNED-LONG)      ;  to distinguish personal and server MSAMs 
   (MailMsgRef :SIGNED-LONG)
   (msgID :IPMMSGID)            ;  kMailLetterIDBit of letter being reported upon 
   (sender (:POINTER :OCERECIPIENT));  sender of the letter you are creating report on 
   )

(defrecord MSAMPutRecipientReportPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailMsgRef :SIGNED-LONG)
   (recipientIndex :SIGNED-INTEGER);  recipient index in the original letter 
   (result :SIGNED-INTEGER)     ;  result of sending the recipient 
;  Values for result:
; 											kIPMNoResponsibility
; 											kIPMNoInformation
; 											kIPMRecMaybeNotSent
; 											kIPMRecMaybeSent
; 										
   )

; **************************************************************************************

(defrecord MailWakeupPMSAMPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (pmsamCID :CREATIONID)
   (MailSlotID :SIGNED-INTEGER)
   )

(defrecord MailCreateMailSlotPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailboxRef :SIGNED-LONG)
   (timeout :SIGNED-LONG)
   (pmsamCID :CREATIONID)
   (SMCA :SMCA)
   )

(defrecord MailModifyMailSlotPB 
   (qLink :POINTER)
   (reservedH1 :SIGNED-LONG)
   (reservedH2 :SIGNED-LONG)
   (ioCompletion :POINTER)
   (ioResult :SIGNED-INTEGER)
   (saveA5 :SIGNED-LONG)
   (reqCode :SIGNED-INTEGER)

   (MailboxRef :SIGNED-LONG)
   (timeout :SIGNED-LONG)
   (pmsamCID :CREATIONID)
   (SMCA :SMCA)
   )

; **************************************************************************************

(defconstant $kPMSAMGetMSAMRecord 1286)
(defconstant $kPMSAMOpenQueues 1280)
(defconstant $kPMSAMLogError 1313)
(defconstant $kPMSAMSetStatus 1319)

(defconstant $kPMSAMCreateMsgSummary 1314)
(defconstant $kPMSAMPutMsgSummary 1317)
(defconstant $kPMSAMGetMsgSummary 1318)

(defconstant $kMailWakeupPMSAM 1287)

(defconstant $kSMSAMSetup 1315)
(defconstant $kSMSAMStartup 1281)
(defconstant $kSMSAMShutdown 1282)

(defconstant $kMSAMEnumerate 1283)
(defconstant $kMSAMDelete 1284)

(defconstant $kMSAMOpen 1288)
(defconstant $kMSAMOpenNested 1289)
(defconstant $kMSAMClose 1290)
(defconstant $kMSAMGetMsgHeader 1297)
(defconstant $kMSAMnMarkRecipients 1298)

(defconstant $kMSAMGetAttributes 1291)
(defconstant $kMSAMGetRecipients 1292)
(defconstant $kMSAMGetContent 1293)
(defconstant $kMSAMGetEnclosure 1294)
(defconstant $kMSAMEnumerateBlocks 1295)
(defconstant $kMSAMGetBlock 1296)
(defconstant $kMSAMMarkRecipients 1285)

(defconstant $kMSAMCreate 1300)
(defconstant $kMSAMBeginNested 1301)
(defconstant $kMSAMEndNested 1302)
(defconstant $kMSAMSubmit 1303)
(defconstant $kMSAMPutMsgHeader 1309)
(defconstant $kMSAMPutAttribute 1304)
(defconstant $kMSAMPutRecipient 1305)
(defconstant $kMSAMPutContent 1306)
(defconstant $kMSAMPutEnclosure 1307)
(defconstant $kMSAMPutBlock 1308)

(defconstant $kMSAMCreateReport 1311)
(defconstant $kMSAMPutRecipientReport 1312)

(defconstant $kMailCreateMailSlot 1323)
(defconstant $kMailModifyMailSlot 1324)

(defrecord MSAMParam 
   (:variant 
      ((header :MAILPARAMBLOCKHEADER))
      ((pmsamGetMSAMRecord :PMSAMGETMSAMRECORDPB))
      ((pmsamOpenQueues :PMSAMOPENQUEUESPB))
      ((pmsamSetStatus :PMSAMSETSTATUSPB))
      ((pmsamLogError :PMSAMLOGERRORPB))
      ((smsamSetup :SMSAMSETUPPB))
      ((smsamStartup :SMSAMSTARTUPPB))
      ((smsamShutdown :SMSAMSHUTDOWNPB))
      ((msamEnumerate :MSAMENUMERATEPB))
      ((msamDelete :MSAMDELETEPB))
      ((msamOpen :MSAMOPENPB))
      ((msamOpenNested :MSAMOPENNESTEDPB))
      ((msamClose :MSAMCLOSEPB))
      ((msamGetMsgHeader :MSAMGETMSGHEADERPB))
      ((msamGetAttributes :MSAMGETATTRIBUTESPB))
      ((msamGetRecipients :MSAMGETRECIPIENTSPB))
      ((msamGetContent :MSAMGETCONTENTPB))
      ((msamGetEnclosure :MSAMGETENCLOSUREPB))
      ((msamEnumerateBlocks :MSAMENUMERATEBLOCKSPB))
      ((msamGetBlock :MSAMGETBLOCKPB))
      ((msamMarkRecipients :MSAMMARKRECIPIENTSPB))
      ((msamnMarkRecipients :MSAMNMARKRECIPIENTSPB))
      ((msamCreate :MSAMCREATEPB))
      ((msamBeginNested :MSAMBEGINNESTEDPB))
      ((msamEndNested :MSAMENDNESTEDPB))
      ((msamSubmit :MSAMSUBMITPB))
      ((msamPutMsgHeader :MSAMPUTMSGHEADERPB))
      ((msamPutAttribute :MSAMPUTATTRIBUTEPB))
      ((msamPutRecipient :MSAMPUTRECIPIENTPB))
      ((msamPutContent :MSAMPUTCONTENTPB))
      ((msamPutEnclosure :MSAMPUTENCLOSUREPB))
      ((msamPutBlock :MSAMPUTBLOCKPB))
      ((msamCreateReport :MSAMCREATEREPORTPB));  Reports and Error Handling Calls 
      ((msamPutRecipientReport :MSAMPUTRECIPIENTREPORTPB))
      ((pmsamCreateMsgSummary :PMSAMCREATEMSGSUMMARYPB))
      ((pmsamPutMsgSummary :PMSAMPUTMSGSUMMARYPB))
      ((pmsamGetMsgSummary :PMSAMGETMSGSUMMARYPB))
      ((wakeupPMSAM :MAILWAKEUPPMSAMPB))
      ((createMailSlot :MAILCREATEMAILSLOTPB))
      ((modifyMailSlot :MAILMODIFYMAILSLOTPB))
      ))


(deftrap _mailcreatemailslot ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (86704384 :signed-longint)))

(deftrap _mailmodifymailslot ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (86769920 :signed-longint)))

(deftrap _maildeletemailslot ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (86835456 :signed-longint)))

(deftrap _mailwakeuppmsam ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (84345088 :signed-longint)))

;  Personal MSAM Glue Routines 

(deftrap _pmsamopenqueues ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (83886080 :signed-longint)))

(deftrap _pmsamsetstatus ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1319 :signed-integer)))

(deftrap _pmsamgetmsamrecord ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (84279296 :signed-longint)))

;  Server MSAM Glue Routines 

(deftrap _smsamstartup ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (83951616 :signed-longint)))

(deftrap _smsamshutdown ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1282 :signed-integer)))

;  Get Interface Glue Routines 

(deftrap _msamenumerate ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1283 :signed-integer)))

(deftrap _msamdelete ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1284 :signed-integer)))

(deftrap _msammarkrecipients ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1285 :signed-integer)))

(deftrap _msamnmarkrecipients ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1298 :signed-integer)))


(deftrap _msamopen ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1288 :signed-integer)))

(deftrap _msamopennested ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1289 :signed-integer)))

(deftrap _msamclose ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1290 :signed-integer)))


(deftrap _msamgetrecipients ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1292 :signed-integer)))

(deftrap _msamgetattributes ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1291 :signed-integer)))

(deftrap _msamgetcontent ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1293 :signed-integer)))

(deftrap _msamgetenclosure ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1294 :signed-integer)))

(deftrap _msamenumerateblocks ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1295 :signed-integer)))

(deftrap _msamgetblock ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1296 :signed-integer)))

(deftrap _msamgetmsgheader ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1297 :signed-integer)))

;  Put Interface Glue Routines 

(deftrap _msamcreate ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1300 :signed-integer)))

(deftrap _msambeginnested ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1301 :signed-integer)))

(deftrap _msamendnested ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (85327872 :signed-longint)))

(deftrap _msamsubmit ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (85393408 :signed-longint)));   SYNCHRONOUS ONLY 


(deftrap _msamputattribute ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1304 :signed-integer)))

(deftrap _msamputrecipient ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1305 :signed-integer)))

(deftrap _msamputcontent ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1306 :signed-integer)))

(deftrap _msamputenclosure ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (85655552 :signed-longint)));   SYNCHRONOUS ONLY 

(deftrap _msamputblock ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1308 :signed-integer)))

(deftrap _msamputmsgheader ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1309 :signed-integer)))

;  Reports and Error Handling Glue Routines 

(deftrap _msamcreatereport ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1311 :signed-integer)))

(deftrap _msamputrecipientreport ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1312 :signed-integer)))

(deftrap _pmsamlogerror ((paramblock (:pointer :msamparam)))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock (86048768 :signed-longint)))

;  MsgSummary Glue Routines 

(deftrap _pmsamcreatemsgsummary ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1314 :signed-integer)))

(deftrap _pmsamputmsgsummary ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1317 :signed-integer)))

(deftrap _pmsamgetmsgsummary ((paramblock (:pointer :msamparam)) (asyncflag :boolean))
   (:stack :signed-integer)
   (:stack-trap #xAA5E paramblock asyncflag (1318 :signed-integer)))

; $ENDC                         ;  UsingOCEMail 

; $IFC NOT UsingIncludes

; $ENDC

(export '($KMAILMODIFYMAILSLOT $KMAILCREATEMAILSLOT $KMSAMPUTRECIPIENTREPORT
          $KMSAMCREATEREPORT $KMSAMPUTBLOCK $KMSAMPUTENCLOSURE $KMSAMPUTCONTENT
          $KMSAMPUTRECIPIENT $KMSAMPUTATTRIBUTE $KMSAMPUTMSGHEADER $KMSAMSUBMIT
          $KMSAMENDNESTED $KMSAMBEGINNESTED $KMSAMCREATE $KMSAMMARKRECIPIENTS
          $KMSAMGETBLOCK $KMSAMENUMERATEBLOCKS $KMSAMGETENCLOSURE
          $KMSAMGETCONTENT $KMSAMGETRECIPIENTS $KMSAMGETATTRIBUTES
          $KMSAMNMARKRECIPIENTS $KMSAMGETMSGHEADER $KMSAMCLOSE $KMSAMOPENNESTED
          $KMSAMOPEN $KMSAMDELETE $KMSAMENUMERATE $KSMSAMSHUTDOWN
          $KSMSAMSTARTUP $KSMSAMSETUP $KMAILWAKEUPPMSAM $KPMSAMGETMSGSUMMARY
          $KPMSAMPUTMSGSUMMARY $KPMSAMCREATEMSGSUMMARY $KPMSAMSETSTATUS
          $KPMSAMLOGERROR $KPMSAMOPENQUEUES $KPMSAMGETMSAMRECORD
          $KMAILRESOLVEDLIST $KSMSAMFWDRMNMSERVERCHANGEDMASK
          $KSMSAMFWDRFOREIGNRLISCHANGEDMASK $KSMSAMFWDRCONNECTEDTOCHANGEDMASK
          $KSMSAMFWDRHOMEINTERNETCHANGEDMASK $KSMSAMFWDREVERYTHINGCHANGEDMASK
          $KSMSAMFWDRMNMSERVERCHANGEDBIT $KSMSAMFWDRFOREIGNRLISCHANGEDBIT
          $KSMSAMFWDRCONNECTEDTOCHANGEDBIT $KSMSAMFWDRHOMEINTERNETCHANGEDBIT
          $KSMSAMGETDYNAMICFWDRPARAMS $KSMSAMNOTIFYFWDRPWDCHANGE
          $KSMSAMNOTIFYFWDRNAMECHANGE $KSMSAMNOTIFYFWDRSETUPCHANGE
          $KMAILMAXPMSAMMSGSUMMARYDATA $KADDRESSEDAS_BCC $KADDRESSEDAS_CC
          $KADDRESSEDAS_TO $KMAILMSGSUMMARYVERSION $KPMSAMSTATUSSENT
          $KPMSAMSTATUSCACHING $KPMSAMSTATUSSENDING $KPMSAMSTATUSERROR
          $KPMSAMSTATUSPENDING $KMAILTIMERFREQUENCY $KMAILTIMERTIME
          $KMAILTIMEROFF $KMAILEPPCLOCATIONCHANGED $KMAILEPPCWAKEUP
          $KMAILEPPCDELETEOUTQMSG $KMAILEPPCMSGOPENED $KMAILEPPCINQUPDATE
          $KMAILEPPCADMIN $KMAILEPPCSCHEDULE $KMAILEPPCCONTINUE
          $KMAILEPPCSENDIMMEDIATE $KMAILEPPCMSGPENDING $KMAILEPPCMAILBOXCLOSED
          $KMAILEPPCMAILBOXOPENED $KMAILEPPCSHUTDOWN $KMAILEPPCDELETESLOT
          $KMAILEPPCMODIFYSLOT $KMAILEPPCCREATESLOT $KMAILEPPCMSGVERSION
          $KOCESETUPLOCATIONMAX $KOCESETUPLOCATIONNONE $KMAILFROMMARK
          $KMAILFROMLEOB $KMAILFROMSTART $KMAILNOMODEM $KMAILMISCERROR
          $KMAILMSAMERRORCODE $KMAILELEINFORMATIONAL $KMAILELEWARNING
          $KMAILELEERROR $KMAILELECORRECTABLE $KMAILMSAMACTIONSTRINGLISTID
          $KMAILMSAMERRORSTRINGLISTID $KMAILERRORLOGENTRYVERSION
          $KMAILMOVIESEGMENTTYPE $KMAILSTYLEDTEXTSEGMENTTYPE
          $KMAILSOUNDSEGMENTTYPE $KMAILPICTSEGMENTTYPE $KMAILTEXTSEGMENTTYPE
          $KMAILINVALIDSEGMENTTYPE $KMAILMOVIESEGMENTMASK
          $KMAILSTYLEDTEXTSEGMENTMASK $KMAILSOUNDSEGMENTMASK
          $KMAILPICTSEGMENTMASK $KMAILTEXTSEGMENTMASK $KMAILMOVIESEGMENTBIT
          $KMAILSTYLEDTEXTSEGMENTBIT $KMAILSOUNDSEGMENTBIT $KMAILPICTSEGMENTBIT
          $KMAILTEXTSEGMENTBIT $KMAILENCLOSEONNONRECEIPT $KMAILNOORIGINAL
          $KMAILORIGINALINREPORTMASK $KMAILNONRECEIPTREPORTSMASK
          $KMAILRECEIPTREPORTSMASK $KMAILFORWARDEDMASK $KMAILPRIORITYMASK
          $KMAILISREPORTWITHORIGINALMASK $KMAILISREPORTMASK
          $KMAILHASCONTENTMASK $KMAILHASSIGNATUREMASK $KMAILAUTHENTICATEDMASK
          $KMAILSENTMASK $KMAILSENTBIT $KMAILAUTHENTICATEDBIT
          $KMAILHASSIGNATUREBIT $KMAILHASCONTENTBIT $KMAILISREPORTBIT
          $KMAILISREPORTWITHORIGINALBIT $KMAILPRIORITYBIT $KMAILFORWARDEDBIT
          $KMAILRECEIPTREPORTSBIT $KMAILNONRECEIPTREPORTSBIT
          $KMAILORIGINALINREPORTBIT $KMAILINTRASHMASK $KMAILDONTARCHIVEMASK
          $KMAILREADMASK $KMAILINTRASHBIT $KMAILDONTARCHIVEBIT $KMAILREADBIT
          $KMAILISLOCALMASK $KMAILISLOCALBIT $KMAILBCCMASK $KMAILCCMASK
          $KMAILTOMASK $KMAILFROMMASK $KMAILSUBJECTMASK
          $KMAILCONVERSATIONIDMASK $KMAILREPLYIDMASK $KMAILMSGFAMILYMASK
          $KMAILNESTINGLEVELMASK $KMAILSENDTIMESTAMPMASK $KMAILLETTERIDMASK
          $KMAILMSGTYPEMASK $KMAILINDICATIONSMASK $KMAILLETTERFLAGSMASK
          $KMAILBCCBIT $KMAILCCBIT $KMAILTOBIT $KMAILFROMBIT $KMAILSUBJECTBIT
          $KMAILCONVERSATIONIDBIT $KMAILREPLYIDBIT $KMAILMSGFAMILYBIT
          $KMAILNESTINGLEVELBIT $KMAILSENDTIMESTAMPBIT $KMAILLETTERIDBIT
          $KMAILMSGTYPEBIT $KMAILINDICATIONSBIT $KMAILLETTERFLAGSBIT
          $KMAILFAMILYFILE $KMAILFAMILY $KMAILREPORTTYPE $KMAILHOPINFOTYPE
          $KMAILTUNNELLTRTYPE $KMAILMSAMTYPE $KMAILIMAGEBODYTYPE
          $KMAILENCLOSUREFILETYPE $KMAILENCLOSUREDESKTOPTYPE
          $KMAILENCLOSURELISTTYPE $KMAILCONTENTTYPE $KMAILLTRHDRTYPE
          $KMAILLTRMSGTYPE $KMAILAPPLEMAILCREATOR))
(provide-interface 'OCEMail)
